import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TBFileHandler {
	private FileHandler fh;
	private String oldHeader;
	private String oldFooter;
	private Pattern oldTags;
	private String newHeader;
	private String newFooter;
	private Pattern newTags;
	private Pattern attributeFilter;
	private boolean changed;
	private Date fileDate;
	private int actualFormatVersion = 2;
	private boolean oldformat = false;
	private boolean plain = false;
	private Date lastedit;
	private int formatVersion;

	public TBFileHandler() {
		this.oldHeader = "<SPEEDPROGLUATB>"; //$NON-NLS-1$
		this.oldFooter = "</SPEEDPROGLUATB>"; //$NON-NLS-1$
		this.newHeader = "<SPDIARY +version=\"(\\d)\" +datum=\"(\\d\\d\\d\\d\\.\\d\\d\\.\\d\\d)\" *(.*?)>"; //$NON-NLS-1$
		this.newFooter = "</SPDIARY>"; //$NON-NLS-1$
		this.oldTags = Pattern
				.compile("(?s)" + this.oldHeader + "(.*)" + this.oldFooter); //$NON-NLS-1$ //$NON-NLS-2$
		this.newTags = Pattern
				.compile("(?s)" + this.newHeader + "(.*)" + this.newFooter); //$NON-NLS-1$ //$NON-NLS-2$
		this.attributeFilter = Pattern.compile("(\\w+)=\"([^\"]+)\""); //$NON-NLS-1$
	}

	public TBFileHandler(Date date) {
		SimpleDateFormat df = new SimpleDateFormat(File.separator
				+ "yyyy" + File.separator + "MM" + File.separator + "dd"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		df.setTimeZone(TimeZone.getDefault());
		String path = df.format(date);
		this.fileDate = date;
		this.lastedit = date;
		this.fh = new FileHandler("." + path + ".tb"); //$NON-NLS-1$ //$NON-NLS-2$
		this.oldHeader = "<SPEEDPROGLUATB>"; //$NON-NLS-1$
		this.oldFooter = "</SPEEDPROGLUATB>"; //$NON-NLS-1$
		this.newHeader = "<SPDIARY +version=\"(\\d)\" +datum=\"(\\d\\d\\d\\d\\.\\d\\d\\.\\d\\d)\" *(.*)>"; //$NON-NLS-1$
		this.newFooter = "</SPDIARY>"; //$NON-NLS-1$
		this.oldTags = Pattern
				.compile("(?s)" + this.oldHeader + "(.*)" + this.oldFooter); //$NON-NLS-1$ //$NON-NLS-2$
		this.newTags = Pattern
				.compile("(?s)" + this.newHeader + "(.*)" + this.newFooter); //$NON-NLS-1$ //$NON-NLS-2$
		this.attributeFilter = Pattern.compile("(\\w+)=\"([^\"]+)\""); //$NON-NLS-1$
	}

	public int getActualFormatVersion() {
		return this.actualFormatVersion;
	}

	public String getFileContend() {
		return this.removeFileHeader(this.fh.getFileContent());
	}

	public Date getFileDate() {
		return this.fileDate;
	}

	public int getFormatVersion() {
		return this.formatVersion;
	}

	public Date getLastedit() {
		return this.lastedit;
	}

	public boolean isChanged() {
		return this.changed;
	}

	public boolean isOldformat() {
		return this.oldformat;
	}

	public boolean isPlain() {
		return this.plain;
	}

	public void readWholeFile() {
		this.fh.readWholeFile();
	}

	public void setFileContend(String text) {
		this.fh.setFileContend(this.addFileHeader(text));

	}

	public void setFilename(Date date) {
		SimpleDateFormat df = new SimpleDateFormat(
				"."		+ File.separator + "yyyy" + File.separator + "MM" + File.separator + "dd"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		df.setTimeZone(TimeZone.getDefault());
		String path = df.format(date);
		this.fileDate = date;
		this.lastedit = date;
		this.fh = new FileHandler(path + ".tb"); //$NON-NLS-1$
	}

	public void writeToFile() {
		if (this.oldformat || this.isPlain()
				|| (this.formatVersion != this.actualFormatVersion))
			this.setFileContend(this.addFileHeader(this
					.removeFileHeader(this.fh.getFileContent())));
		this.fh.writeToFile();
	}

	private String addFileHeader(String fileContent) {
		SimpleDateFormat df = new SimpleDateFormat("yyyy.MM.dd"); //$NON-NLS-1$
		df.setTimeZone(TimeZone.getDefault());
		String datum = df.format(this.fileDate);
		df = new SimpleDateFormat("yyyy-MM-dd"); //$NON-NLS-1$
		df.setTimeZone(TimeZone.getDefault());
		this.plain = false;
		this.oldformat = false;
		this.formatVersion = this.actualFormatVersion;
		this.lastedit = new Date();
		return "<SPDIARY version=\"" + String.valueOf(this.actualFormatVersion) + "\" datum=\"" //$NON-NLS-1$ //$NON-NLS-2$
				+ datum
				+ "\" lastedit=\"" + df.format(this.lastedit) + "\">" + fileContent + this.newFooter; //$NON-NLS-1$ //$NON-NLS-2$
	}

	private String removeFileHeader(String fileContend) {
		String retC;
		Matcher tm = this.oldTags.matcher(fileContend);
		if (tm.find()) {
			this.plain = false;
			this.oldformat = true;
			retC = tm.group(1);
			this.formatVersion = 1;
			this.lastedit = this.fileDate;
		} else {
			tm = this.newTags.matcher(fileContend);
			if (tm.find()) {
				this.plain = false;
				this.oldformat = false;
				this.formatVersion = Integer.parseInt(tm.group(1));
				retC = tm.group(4);
				if (!(tm.group(3).trim().equals(""))) //$NON-NLS-1$
					for (Matcher am = this.attributeFilter.matcher(tm.group(3)); am
							.find();)
						if (am.group(1).equals("lastedit")) { //$NON-NLS-1$
							SimpleDateFormat df = new SimpleDateFormat(
									"yyyy-MM-dd"); //$NON-NLS-1$
							try {
								this.lastedit = df.parse(am.group(2));
							} catch (ParseException e) {
								this.lastedit = new Date();
							}
						}
			} else {
				this.oldformat = false;
				this.plain = true;
				retC = fileContend;
			}
		}
		return retC;
	}

}
